// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *ABookGetBasePtr ()
{
	return (gr_Agenda->DataFile.DataPtr + gr_Agenda->DataFile.Header->AddressbookPtr);
}

// --------------------------------------------------------------------------

unsigned int ABookGetBaseSize ()
{
	return (gr_Agenda->DataFile.Header->AddressbookSize);
}

// --------------------------------------------------------------------------

void AdjustABookSize (long int SizeDelta)
{
	gr_Agenda->DataFile.Header->AddressbookSize += SizeDelta;
	gr_Agenda->DataFile.Header->NotesPtr += SizeDelta;
	gr_Agenda->DataFile.Header->INetLinksPtr += SizeDelta;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetAddressNames (TEntryType *ET, int GroupNr, HANDLE *handle)
{
	int				Ret			= ER_SUCCESS;
	unsigned long	size		= 0;
	unsigned int	dPos		= 0;
	char			*pos;
	long			dSize;
	char			*lastname;
	char			*firstname;
	TEntryHeader	*Header;
	
	*handle = HeapAlloc (2);
	if (*handle == H_NULL) Ret = ER_MALLOC;
	
	if (Ret == ER_SUCCESS)
	{
		Ret = GetGroup (ET, NULL, GroupNr, &Header);
		if (Ret == ER_SUCCESS)
		{
			Header = (TEntryHeader *)( (char *)Header + Header->Size);
			while ( ((Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY) && (Ret == ER_SUCCESS) )
			{
				lastname = &Header->data;
				firstname = GetStrFromListEx (&Header->data, 1);
				dSize = strlen (lastname) + strlen (firstname) + 2;
				if  ( HeapRealloc (*handle, size += dSize) == H_NULL ) Ret = ER_MALLOC;
				else
				{
					pos = HeapDeref (*handle) + dPos;
					strcpy (pos, lastname);
					strcat (pos, " ");
					strcat (pos, firstname);
					dPos += dSize;
				}
				Header = (TEntryHeader *)( (char *)Header + Header->Size);
			}
		}
		poke_w_ex (HeapDeref (*handle) + dPos, 0);
		if (Ret == ER_GROUP_NOT_FOUND) Ret = ER_SUCCESS;
	}
	
	if (Ret == ER_MALLOC) HeapFreeHNULL (handle);

	return Ret;

}

int GetBirthdays (unsigned int Day, HANDLE *hStringList, HANDLE *hEPList)
{
	// Returns in *handle a string list with the names, at which a birthday
	// on day Day is registred. 
	
	int				Ret				= ER_SUCCESS;
	TEntryHeader	*Header;
	char			*birth;
	TDate			dateOfDay;
	TDate			*dateOfBirth;
	HANDLE			hDatesOfBirth	= H_NULL;
	int				dpDateOfBirth;
	int				size			= 2;
	long			dSize;
	char			*firstname;
	char			*lastname;
	int				dPos			= 0;
	char			*ptr;
	int				GroupNr			= -1;
	int				DatasetNr		= DatasetNr;	// --- will be set to -1 after first group (which is always first entry)
	long			hEPSize			= 2;			// --- Size of the hEntryPos handle
	
	GetDate (Day, &dateOfDay);
	Header = (TEntryHeader *)ABookGetBasePtr();

	*hStringList = HeapAlloc (2);
	*hEPList = HeapAlloc (2);
	if ( (hStringList == NULL) || (hEPList == NULL) ) Ret = ER_MALLOC;

	while ( (Header->Flags != FLAG_ET_DATAEND) && (Ret == ER_SUCCESS) )
	{
		if ((Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY)
		{
			// --- next Dataset (for getting the entry position)
			DatasetNr++;

			birth = GetStrFromListEx (&Header->data, 7);
			if (*birth != 0)
			{
				Ret = GetDayFromStrDateEx (birth, &hDatesOfBirth, 0);
				dpDateOfBirth = 0;
				while (Ret == ER_SUCCESS && (*((int *)(HeapDeref (hDatesOfBirth) + dpDateOfBirth)) != DATE_ENDLIST) )
				{
					dateOfBirth = HeapDeref (hDatesOfBirth) + dpDateOfBirth;
					if (dateOfBirth->MonthDay == dateOfDay.MonthDay &&
					    dateOfBirth->Month == dateOfDay.Month)
					{
						lastname = &Header->data;
						firstname = GetStrFromListEx (&Header->data, 1);
						dSize = 2 + strlen (firstname) + strlen (lastname);

						if ( (HeapRealloc (*hStringList, size += dSize) == H_NULL) ||
						     (HeapRealloc (*hEPList, hEPSize +4) == H_NULL)) Ret = ER_MALLOC;
						else
						{
							ptr = HeapDeref (*hStringList) + dPos;
							strcpy (ptr, lastname);
							strcat (ptr, " ");
							strcat (ptr, firstname);
							dPos += dSize;
							
							// --- storing the position of this found entry in the appropriate list
							ptr = HeapDeref (*hEPList) + hEPSize - 2;
							poke_w (ptr, GroupNr);
							poke_w (ptr+2, DatasetNr);
							hEPSize += 4;
							
						}
					}
					dpDateOfBirth += sizeof (TDate);
				}
				HeapFreeHNULL (&hDatesOfBirth);
			}
		}
		else
		{
			// --- No Entry --> is a Group (or End) so increment the group counter...
			GroupNr++;
			DatasetNr = -1;
		}

		Header = (TEntryHeader *)( (char *)Header + Header->Size);
	}
	
	if (Ret == ER_SUCCESS) poke_w_ex (HeapDeref (*hStringList) + dPos, 0);			// end of string list
	
	return Ret;
}

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
