// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *AgendaGetBasePtr ()
{
	return (gr_Agenda->DataFile.DataPtr + gr_Agenda->DataFile.Header->AgendaPtr);
}

// --------------------------------------------------------------------------

void AdjustAgendaSize (long int SizeDelta)
{
	gr_Agenda->DataFile.Header->AgendaSize += SizeDelta;
	gr_Agenda->DataFile.Header->AutofieldPtr += SizeDelta;
	gr_Agenda->DataFile.Header->AddressbookPtr += SizeDelta;
	gr_Agenda->DataFile.Header->NotesPtr += SizeDelta;
	gr_Agenda->DataFile.Header->INetLinksPtr += SizeDelta;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Writes the data of a day to agnddata ---------------------------------
// --------------------------------------------------------------------------

int WriteDay (unsigned int Day, HANDLE hData)
{
	int				Ret 		= ER_SUCCESS;
	long int		dSize		= 0;
	char			*EntryPos;
	char			*NewData;
	TAgendaHeader	*Header;
	
	if ( strcmp (HeapDeref (hData), ReadDay (Day, NULL)) != 0 )
	{
		Ret = DeleteDay (Day);
		if ( strlen (HeapDeref (hData)) != 0)
		{
			if (Ret == ER_SUCCESS)
			{
				EntryPos = gr_Agenda->DataFile.DataPtr + gr_Agenda->DataFile.Header->AgendaPtr + gr_Agenda->DataFile.Header->AgendaSize - 2;
				dSize = strlen (HeapDeref (hData)) +1 +sizeof(TAgendaHeader)-2;
				dSize = RoundUpEven (dSize);
				Ret = ReallocDataFile (EntryPos, gr_Agenda->DataFile.Header->Size +2 + dSize, &EntryPos);
			}
		
			if (Ret == ER_SUCCESS)
			{
				NewData = HeapDeref (hData);
				Header = (TAgendaHeader *)EntryPos;
				memset (Header, 0, sizeof (TAgendaHeader)-2);
				Header->Flags = FLAG_ET_AGENDA | FLAG_ET_ENTRY;
				Header->Shortcut = 0;
				Header->Size = (unsigned int)dSize;
				Header->Day = Day;
				strcpy (&Header->data, NewData);
				AdjustAgendaSize (dSize);
			}
		}
	}
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Returns a pointer to the data of a day -------------------------------
// --------------------------------------------------------------------------

char *ReadDay (unsigned int Day, TAgendaHeader **AgndHeader)
{
	TAgendaHeader	*Header		= (TAgendaHeader *)AgendaGetBasePtr();
	int				found		= 0;
	char			*DataPtr	= NULL;
	
	while ( (Header->Flags != FLAG_ET_DATAEND) && !found)
	{
		if (Header->Day == Day && ((Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY) )
		{
			if (AgndHeader != NULL) *AgndHeader = Header;
			DataPtr = &Header->data;
			found = 1;
		}
		else Header = (TAgendaHeader *)((char *)Header + Header->Size);
	}
	
	return DataPtr;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Deletes a day from agnddata ------------------------------------------
// --------------------------------------------------------------------------

int DeleteDay (unsigned int Day)
{
	int				Ret 		= ER_SUCCESS;
	TAgendaHeader	*Header;
	char			*DataPos;
	long			dSize;
	
	DataPos = ReadDay (Day, NULL);
	if (DataPos != NULL)
	{
		Header = (TAgendaHeader *)(DataPos - ( sizeof(TAgendaHeader) -2));
		dSize =  (unsigned int)Header->Size;
		dSize *= (-1);
		Ret = ReallocDataFile ((char *)Header, gr_Agenda->DataFile.Header->Size +2 + dSize, NULL);
		if (Ret == ER_SUCCESS) AdjustAgendaSize (dSize);
	}
	
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int	DeleteAgenda ()
{
	int			Ret		= ER_SUCCESS;
	long		dSize;
	
	dSize = (unsigned int)gr_Agenda->DataFile.Header->AgendaSize - 2;
	dSize *= (-1);
	Ret = ReallocDataFile ((gr_Agenda->DataFile.DataPtr + gr_Agenda->DataFile.Header->AgendaPtr), gr_Agenda->DataFile.Header->Size +2 +dSize, NULL);
	
	if (Ret == ER_SUCCESS) AdjustAgendaSize (dSize);
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

unsigned int ReadDayNumber ()
{
	unsigned int	Day;
	
	Day = gr_Agenda->DataFile.Header->LastAgendaDay;
	if (Day < MIN_DAY) Day = MIN_DAY;
	if (Day > MAX_DAY) Day = MAX_DAY;
	
	return Day;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
