// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *LVGetHeadTitle (TEntryType *ET, int GroupNr, char *Buf)
{
// size of Buf[] must be at least 30 chars!

	int				Ret 		= ER_SUCCESS;
	TEntryHeader	*Header;

	strcpy (Buf, ET->TaskTitle);
	if (GroupNr != -1)
	{
		Ret = GetGroup (ET, NULL, GroupNr, &Header);
		if (Ret == ER_SUCCESS)
		{
			strcat (Buf, " - ");
			strncat (Buf, &Header->data, 29);
		}
	}

	return Buf;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVFocusGroups (TEntryType *ET, TListView *LV)
{
	int		Ret = ER_SUCCESS;
	
	ScrollList (&LV->SLGroups, NULL, 0);
	LV->key = LV->SLGroups.key;
	LV->GroupNr = LV->SLGroups.CursorIndex + LV->SLGroups.ListIndex;
	LV->Reassign |= 0b10;
	LVReassignData (ET, LV);
	LV->Redraw |= DA_REDRAW_LN;
	if ( !LV->noDataset && (LV->key == KEY_RIGHT || LV->key == KEY_ENTER) )
	{
		LV->Focus = 2;
		LV->DSCursor = CU_NORMAL;
		LV->GroupCursor = CU_LOSTFOCUS;
		LV->DatasetNr = 0;
		LV->SLNames.CursorIndex = 0;
		LV->SLNames.ListIndex = 0;
		LVRedrawLists (LV, 0b01);
		LV->key = 0;
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVFocusNames (TEntryType *ET, TListView *LV)
{
	int		Ret = ER_SUCCESS;
	
	ScrollList (&LV->SLNames, NULL, 0);
	LV->key = LV->SLNames.key;
	LV->DatasetNr = LV->SLNames.CursorIndex + LV->SLNames.ListIndex;
	if (LV->key == KEY_LEFT)
	{
		LV->Focus = 1;
		LV->GroupCursor = CU_NORMAL;
		LV->DSCursor = CU_NONE;
		LV->DatasetNr = 0;
		LV->SLNames.CursorIndex = 0;
		LV->SLNames.ListIndex = 0;
		LVRedrawLists (LV, 0b10); 
		LV->key = 0;
	}
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void LVCheckData (TListView *LV, int Check)
{
	if (Check & 1)
	{
		if ( peek (HeapDeref (LV->hGroups)) == '\0')
		{
			LV->noGroup = 1;
			LV->GroupNr = 0;
			LV->Focus = 0;
			LV->GroupCursor = CU_NONE;
			LV->DSCursor = CU_NONE;
		}
		else
		{
			LV->noGroup = 0;
			if (LV->Focus == 0)
			{
				LV->Focus = 1;
				LV->GroupCursor = CU_NORMAL;
			}
		}
	}

	if (Check & 2)
	{
		if ( peek (HeapDeref (LV->hNames)) == '\0')
		{
			LV->noDataset = 1;
			LV->DatasetNr = 0;
			if (LV->Focus == 2)
			{
				LV->Focus = 1;
				LV->DSCursor = CU_NONE;
			}
		}
		else
		{
			LV->noDataset = 0;
		}
	}	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVReassignData (TEntryType *ET, TListView *LV)
{
	int		Ret 		= ER_SUCCESS;
	int		reassign	= 0;
	
	if (LV->Reassign & 1)
	{
		HeapFreeHNULL (&LV->hGroups);
		Ret = GetGroups (ET, &LV->hGroups, -1, 0);
		if (Ret == ER_SUCCESS)
		{
			LV->Reassign &= ~1;
			reassign |= 1;
		}
	}
	
	if ( (Ret == ER_SUCCESS) && (LV->Reassign & 2) )
	{
		HeapFreeHNULL (&LV->hNames);
		Ret = ET->GetDatasetList (ET, LV->GroupNr, &LV->hNames);
		if (Ret == ER_SUCCESS)
		{
			LV->Reassign &= ~2;
			reassign |= 2;
		}
	}

	if ( (Ret == ER_SUCCESS) && reassign)
	{
		if (reassign & 1) ScrollListReassignData (&LV->SLGroups, HeapDeref (LV->hGroups));
		if (reassign & 2) ScrollListReassignData (&LV->SLNames, HeapDeref (LV->hNames));
		LVCheckData (LV, reassign);
	}		
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void LVRedrawLists (TListView *LV, short Lists)
{
	if (Lists & 1) ScrollListRedraw (&LV->SLGroups, LV->GroupCursor);
	if (Lists & 2) ScrollListRedraw (&LV->SLNames, LV->DSCursor);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void LVRedraw (TListView *LV, TEntryType *ET)
{
	short	Lists;
	
	if (LV->Redraw & DA_REDRAW_INIT)
	{
		LV->Redraw &= ~DA_REDRAW_INIT;
		EmptyScreenClient();
		FontSetSys (F_6x8);
		PrintHeadBox (ET->TaskTitle);
		DrawStrXY (10, 20, GROUP_LIST, A_REPLACE);
		DrawStrXY (93, 20, ET->DatasetNames, A_REPLACE);
		DrawLine (10, 28, 230, 28, A_NORMAL);
		DrawLine (88, 28, 88, 110, A_NORMAL);
	}

	if (LV->Redraw)
	{
		Lists = LV->Redraw & (DA_REDRAW_LG | DA_REDRAW_LN);
		Lists >>= DA_REDRAW_SHR_L; 
		LVRedrawLists (LV, Lists);
		LV->Redraw &= ~(DA_REDRAW_LG | DA_REDRAW_LN);
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVonMenuOpen (void *ptr1, void *ptr2, int *Tag1, int *Tag2)
{
	TListView		*LV;
	
	LV = (TListView *)ptr1;
	*Tag1 = LV->GroupCursor;
	*Tag2 = LV->DSCursor;

	LV->GroupCursor = CU_LOSTFOCUS;
	if (LV->Focus == 2) LV->DSCursor = CU_LOSTFOCUS;
	ScrollListCursorOff (&LV->SLGroups);
	ScrollListCursorOff (&LV->SLNames);
	LVRedrawLists (LV, 0b11);

	return ER_SUCCESS;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVonMenuClose (void *ptr1, void *ptr2, int Tag1, int Tag2, int DidWhat, int Feedback)
{
	TListView		*LV;
	
	LV = (TListView *)ptr1;

	ScrollListCursorOff (&LV->SLGroups);
	ScrollListCursorOff (&LV->SLNames);
	LV->GroupCursor = Tag1;
	LV->DSCursor = Tag2;
	LV->Reassign = 0b11;
	LV->Redraw = DA_REDRAW_LG | DA_REDRAW_LN;
	
	if ( (DidWhat == RET_MM_RESTORE_BACKUP) && (Feedback == ER_SUCCESS))
	{
		LV->GroupNr = 0;
		LV->DatasetNr = 0;
		LV->SLGroups.CursorIndex = 0;
		LV->SLGroups.ListIndex = 0;
		LV->SLNames.CursorIndex = 0;
		LV->SLNames.ListIndex = 0;
		LV->Focus = 1;
		LV->GroupCursor = CU_NORMAL;
		LV->DSCursor = CU_NONE;
	}

	return ER_SUCCESS;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int LVInit (TEntryType *ET, TListView *LV)
{
	int		Ret = ER_SUCCESS;
		
	memset (LV, 0, sizeof(TListView));

	LV->Redraw = DA_REDRAW_INIT | DA_REDRAW_LG | DA_REDRAW_LN;
	LV->Reassign = 0b11;

	LV->GroupCursor = CU_NORMAL;
	LV->GroupNr = 0;

	LV->DSCursor = CU_NONE;
	LV->DatasetNr = 0;

	FontSetSys (F_6x8);

	LV->SLGroups.CursorIndex = LV->GroupNr;
	LV->SLGroups = InitScrollList (ET->Task, 10, 32, 12, 8, NULL, 0, 0, SL_DONT_CLEAR | SL_CALLBACK | SL_HINT_IF_EMPTY);  
	LV->SLGroups.idl.BeforeMenuHook = &LVonMenuOpen;
	LV->SLGroups.idl.AfterMenuHook = &LVonMenuClose;
	LV->SLGroups.idl.Ptr1 = LV;
	
	LV->SLNames.CursorIndex = LV->DatasetNr;
	LV->SLNames = InitScrollList (ET->Task, 93, 32, 23, 8, NULL, 0, 0, SL_DONT_CLEAR | SL_CALLBACK | SL_HINT_IF_EMPTY);  
	LV->SLNames.idl.BeforeMenuHook = &LVonMenuOpen;
	LV->SLNames.idl.AfterMenuHook = &LVonMenuClose;
	LV->SLNames.idl.Ptr1 = LV;
	
	Ret = LVReassignData (ET, LV);

	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int ShowListView (TEntryType *ET, int *groupNr, int *datasetNr, int *mode)
{
	int				Ret					= ER_SUCCESS;
	TListView		LV;
	int				DSCursor;
	int				GroupCursor;
	int				DidWhat				= 0;
	unsigned int	lastMode;
	
	LVInit (ET, &LV);
	
	LV.GroupNr = ScrollListSetCursor (&LV.SLGroups, *groupNr);
	LV.Reassign |= 0b11;	// reassign names; reassign groups if no group detected in
							// LVCheckData below
	if (*datasetNr != -1)
	{
		LV.Focus = 2;
		LV.GroupCursor = CU_LOSTFOCUS;
		LV.DatasetNr = ScrollListSetCursor (&LV.SLNames, *datasetNr);
	}	
	LVCheckData (&LV, 0b11);

	while ( (Ret == ER_SUCCESS) && (LV.key != KEY_ESC) && (!gr_Agenda->Exit))
	{
		Ret = LVReassignData (ET, &LV);
		if (Ret == ER_SUCCESS)
		{
			LVRedraw (&LV, ET);
			switch (LV.Focus)
			{
				case 1:
					Ret = LVFocusGroups (ET, &LV);
					break;
					
				case 2:
					Ret = LVFocusNames (ET, &LV);
					break;
				case 0:
					IdleLoop (&LV.SLGroups.idl, 0, &LV.key, NULL, NULL);
			}
		}

		if ( (Ret == ER_SUCCESS) && (IsEventKey (&LV, ET) ) )
		{
			GroupCursor = LV.GroupCursor;
			DSCursor = LV.DSCursor;
			LV.GroupCursor = CU_LOSTFOCUS;
			if (LV.Focus == 2) LV.DSCursor = CU_LOSTFOCUS;
			ScrollListCursorOff (&LV.SLGroups);
			ScrollListCursorOff (&LV.SLNames);
			LVRedrawLists (&LV, 0b11);

			Ret = ListViewMenu (ET, LV.Focus, LV.GroupNr, LV.DatasetNr, LV.key, &DidWhat);

			if (DidWhat != RET_LV_DS_ADD)
			{
				ScrollListCursorOff (&LV.SLGroups);
				ScrollListCursorOff (&LV.SLNames);
			}

			LV.GroupCursor = GroupCursor;
			LV.DSCursor = DSCursor;
			LV.Reassign = 0b11;
			LV.Redraw = DA_REDRAW_LG | DA_REDRAW_LN;

			if ( (DidWhat == RET_LV_DS_ADD) ||
			     (DidWhat == RET_LV_DS_EDIT) ) LV.Redraw |= DA_REDRAW_INIT;

			if (DidWhat == RET_LV_GM_DEL && Ret == ER_SUCCESS)
			{
				if ( ( (LV.GroupNr +1) == LV.SLGroups.NumLines) &&
				     (LV.GroupNr > 0) )
				{
					LV.GroupNr--;
				}
			}

			if (DidWhat == RET_LV_GM_ADD && Ret == ER_SUCCESS)
			{
				LV.Focus = 1;
				LV.DSCursor = CU_NONE;
				LV.GroupNr = GetGroupCount (ET);
				LV.SLGroups.CursorIndex = LV.GroupNr;
				LV.SLGroups.ListIndex = 0;
			}

			if (DidWhat == RET_LV_DS_ADD && Ret == ER_SUCCESS)
			{
				LV.Focus = 2;
				LV.DSCursor = CU_NORMAL;
				LV.GroupCursor = CU_LOSTFOCUS;
				Ret = GetDatasetCount (ET, LV.GroupNr, &LV.DatasetNr);
				LV.SLNames.CursorIndex = LV.DatasetNr;
				LV.SLNames.ListIndex = 0;
			}
			
			if (DidWhat == RET_LV_DS_MOVE_UP && Ret == ER_SUCCESS)
			{
				if (LV.SLNames.CursorIndex == 0) LV.SLNames.ListIndex--;
				else LV.SLNames.CursorIndex--;
			}
			if (DidWhat == RET_LV_DS_MOVE_DOWN && Ret == ER_SUCCESS)
			{
				if ( LV.SLNames.CursorIndex == LV.SLNames.rows-1) LV.SLNames.ListIndex++;
				else LV.SLNames.CursorIndex++;
			}					
			
			if (Ret == CANCEL) Ret = ER_SUCCESS;

		}
		
	}
	
	//	--- directly write the last positions to the data file
	getLastState (ET->Task, NULL, NULL, &lastMode);
	lastMode = (lastMode & ~MASK_VIEW) | (LV.Focus == 2);
	setLastState (ET->Task, &LV.GroupNr, &LV.DatasetNr, &lastMode);
		
	HeapFreeHNULL (&LV.hNames);
	HeapFreeHNULL (&LV.hGroups);

	return Ret;
}

// --------------------------------------------------------------------------

int LVView (int Task)
{
	int				Ret = ER_SUCCESS;
	int				groupNr;
	int				datasetNr;
	int				mode;
	TEntryType		ET;
	
	InitEntryType (&ET, Task);

	if (!gr_Agenda->TaskQueue.keepTask) EmptyScreenClient();
	if (gr_Agenda->TaskQueue.prm0 != TASK_NO_MESSAGE)
	{
		mode = gr_Agenda->TaskQueue.NextMode;
		groupNr = gr_Agenda->TaskQueue.prm0;
		datasetNr = gr_Agenda->TaskQueue.prm1;
	}
	else
	{
		getLastState (ET.Task, &groupNr, &datasetNr, &mode);

		if (mode == 0) datasetNr = -1;
	}
	
	resetTaskQueueParams();

	if (mode & EDITOR_VIEW)
	{
		Ret = MenuEditDataset (&ET, groupNr, datasetNr); 
	}

	if ( (Ret == ER_SUCCESS) && !gr_Agenda->Exit)
	{
		Ret = ShowListView (&ET, &groupNr, &datasetNr, &mode);
	}
	
	// --- since the pointers below become illegal after reallocation of datafile
	// --- they are relookedup again before saving last position.
	// --- fixme: why?
	
	InitEntryType (&ET, Task);

	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
