// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MainMenu (int key, int *DidWhat)
// gets only triggered from IdleLoop. The return value is only to overgive the
// feedback var in the OnMenuClose hook. Is used to see, whether the Backup has
// successfully restored or not. (yet only for this.)
{
	int			Ret		= ER_SUCCESS;
	int			idx		= 0;
	HANDLE 		handle;

	if (key == KEY_MODE)
	{
		if ( (handle = PopupNew ((char *)MM_TITLE, 100)) == H_NULL) Ret = ER_MALLOC;
		else
		{
			PopupAddText (handle, -1, MM_CONFIG, RET_MM_CONFIG);
			PopupAddText (handle, -1, MM_PASSWORD, RET_MM_PASSWORD);
			PopupAddText (handle, -1, MM_MAKE_BACKUP, RET_MM_MAKE_BACKUP);
			PopupAddText (handle, -1, MM_RESTORE_BACKUP, RET_MM_RESTORE_BACKUP);
			PopupAddText (handle, -1, MM_INFO, RET_MM_INFO);
			
			if (MenuFlags (handle) & MF_ERROR) Ret = ER_MALLOC;			
			else
			{
				idx = PopupDo (handle, CENTER, CENTER, 0);
				
				stIdle();
			}
			HeapFree (handle);
		}
	}
	
	if (key == KEY_F6) idx = RET_MM_CONFIG;
	if (key == KEY_F7) idx = RET_MM_MAKE_BACKUP;
	if (key == KEY_F8) idx = RET_MM_RESTORE_BACKUP; 

	if (Ret == ER_SUCCESS)
	{
		switch (idx)
		{
			case RET_MM_MAKE_BACKUP:
					Ret = MenuMakeBackup ();
					break;

			case RET_MM_RESTORE_BACKUP:
					Ret = MenuRestoreBackup ();
					break;

			case RET_MM_INFO:
					MenuInfo ();
					break;

			case RET_MM_CONFIG:
					Ret = MenuConfig ();
					break;
			
			case RET_MM_PASSWORD:
					Ret = MenuChangePassword ();
					break;
		}
	
		if (DidWhat != NULL) *DidWhat = idx;
	}


	ShowErrors (Ret);

	return Ret;
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MenuMakeBackup ()
{
	int			Ret		= ER_SUCCESS;
	
	if ( (Ret = MsgBox (MSG_TITLE_HINT, MSG_MK_BKP_ASK, BT_YES, BT_NO)) == OK)
	{	
		Ret = MakeBackup ();
		if (Ret == ER_SUCCESS)
		{
			MsgBox ( MSG_TITLE_HINT, MSG_MAKE_BACKUP, BT_OK, BT_NONE);
		}
	}
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MenuRestoreBackup ()
{
	int			Ret = ER_SUCCESS;
	
	if ( (Ret = MsgBox (MSG_TITLE_HINT, MSG_RST_BKP_ASK, BT_YES, BT_NO)) == OK )
	{
		Ret = RestoreBackup ();
		if (Ret == ER_SUCCESS)
		{
			MsgBox ( MSG_TITLE_HINT, MSG_RST_BACKUP, BT_OK, BT_NONE);
		}
	}	

	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


int	 getPublicDayData (HANDLE *handle)
{
	int				Ret 		= ER_SUCCESS;
	int				size 		= 0;
	char			*pSource	= PUBLIC_HOLIDAYS;
	char			*pDest		= NULL;
	long			dSize;
	unsigned long	flags		= gr_Agenda->DataFile.Header->SpecialDays;
	
	if ( (*handle = HeapAlloc (6)) == H_NULL) Ret = ER_MALLOC;

	while ( (*pSource != 0) && (Ret == ER_SUCCESS) )
	{
		dSize = strlen (pSource) + 3;
		if (HeapRealloc (*handle, size + dSize +1) == H_NULL) Ret = ER_MALLOC;
		else
		{
			pDest = HeapDeref (*handle);
			strcpy (pDest + size, "  ");
			strcat (pDest + size, pSource);
			
			if (! (flags & 1)) poke (pDest + size,15);
			flags = flags >> 1;

			size += dSize;
			pSource += dSize - 2; 		
			
		}		
	}
	
	if (Ret == ER_SUCCESS) poke (pDest + size, 0);
		
	return Ret;
}


void savePublicDayOptions (char *data)
{
	unsigned int	i				= 0;
	unsigned long int	newFlags	= 0;
	
	while (*data != 0)
	{
		if ( *data != 15) newFlags |= (1L << i);
 		data += strlen (data) + 1;
		i++;
	}
	
	gr_Agenda->DataFile.Header->SpecialDays = newFlags;
}


void PublicDaysAll (char *data, int state)
{
	char	ch;
	
	if (state) ch = 15;			// check mark ^= #15
	else ch = ' ';
	
	while (*data != 0)
	{
		*data = ch;
 		data += strlen (data) + 1;
	}
}

void ConfigPublicDays ()
{
	int				Ret 	= ER_SUCCESS;
	WINDOW 			*Win	= HeapAllocPtr (sizeof (WINDOW));
	HANDLE			handle	= H_NULL;
	char			*pData;
	int				idx;
	TScrollList		SL;
	char			*pTick;
	
	if (WinOpen (Win, MakeWinRect (20,18,220,111), WF_SAVE_SCR | WF_ROUNDEDBORDER | WF_TITLE, MPH_TITLE))
	{
		WinActivate (Win);
		WinFont (Win, F_6x8);
		DrawStaticButton (Win, BT_OK, 7);
		DrawStaticButton (Win, BT_CANCEL, 115);
		stIdle();

		ST_helpMsg (TEXT_PH_KEYS);
		Ret = getPublicDayData (&handle);
		if (Ret == ER_SUCCESS)
		{
			HeapLock (handle);
			pData = HeapDeref (handle);
			SL = InitScrollList (TASK_TASKMENU, 30, 35, 30, 6, pData, 0, 0, SL_CALLBACK | SL_DONT_CLEAR);
			while (! (SL.key == KEY_ENTER || SL.key == KEY_ESC) )
			{
				ScrollList (&SL, &idx, ILF_NO_TASK_SWITCH | ILF_NO_EXIT | ILF_NOMENU | ILF_NO_OFF_ON_EXIT);
				if (SL.key == ' ')
				{
					pTick = GetStrFromList (pData, idx); 
					if (*pTick == ' ') *pTick = 15;
					else *pTick = ' ';
					SL.Redraw = 1;
				}
				if (SL.key == KEY_F1 || SL.key == KEY_F2)
				{
					PublicDaysAll (pData, SL.key == KEY_F1);
					SL.Redraw = 1;
				}
			}
			
			if (SL.key == KEY_ENTER) savePublicDayOptions (pData);
		}
		else ShowErrors (Ret);				
		
		WinClose (Win);
		HeapFreePtr (Win);
	}
	
	HeapFreeHNULL (&handle);

}

short CALLBACK OnMenuConfigChange (short x, long y)
{
	void			*oldA5;
	TAgenda			*oldA3;
	
	// --- restore a5 with jump table base (OPTIMIZE_ROM_CALLS)
	// --- and restore a3, the global Agenda struct pointer
	oldA3 = gr_Agenda;
	gr_Agenda = g_Agenda;

	#ifdef USE_OPT_ROM__CALLS
		oldA5 = __jmp_tbl;
		__jmp_tbl = *(void **)0xC8;
	#endif
	
	if (x == 4)
	{
		ConfigPublicDays ();
	}

	// --- restore original a5 and a3 with the values they had on entrance

	#ifdef USE_OPT_ROM__CALLS
		__jmp_tbl = oldA5;
	#endif
	
	gr_Agenda = oldA3;

	return 1;
}


int	MenuConfig ()
{
	HANDLE			hPopUpPW		= H_NULL;
	HANDLE			hPopUpAB		= H_NULL;
	HANDLE			hPopUpDF		= H_NULL;
	HANDLE			hDialog			= H_NULL;
	HANDLE			hPopupPD		= H_NULL;
	int				Ret				= ER_SUCCESS;
	int				PDBuffer[]		= {2,2,0,0};
	int				oldDateformat	= readDateFormat();
	unsigned int	Flags;
		
	Ret = MakePopup (NULL, MC_OP_YESNO, 0, &hPopUpPW) |
	      MakePopup (NULL, MC_OP_AB, 0, &hPopUpAB) |
	      MakePopup (NULL, (char *)"", 0, &hPopupPD) |
	      MakePopup (NULL, MC_OP_DATEFORM, 0, &hPopUpDF);
	
	if (Ret == ER_SUCCESS)
	{	
		hDialog = DialogNew (195,80, &OnMenuConfigChange);
		if (hDialog == H_NULL) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)	
	{
		Flags = GetFlags();
		if (Flags & FLAG_PASSWORD) PDBuffer[0] = 1;
		Flags &= FLAGS_AB;
		Flags >>= FLAG_AB_OFFSET;
		PDBuffer[1] = Flags+1;
		PDBuffer[2] = oldDateformat;
		if ( ( DialogAddTitle (hDialog, MC_TITLE, BT_OK, BT_CANCEL) &&
			   DialogAddPulldown (hDialog, 10, 20, MC_USEPW, hPopUpPW, 0) &&
			   DialogAddPulldown (hDialog, 10, 30, MC_AUTOBACKUP, hPopUpAB, 1) &&
			   DialogAddPulldown (hDialog, 10, 40, MC_DATEFORMAT, hPopUpDF, 2) ) &&
			   DialogAddPulldown (hDialog, 10, 50, MC_PDSETUP, hPopupPD, 3) == 0
		    ) Ret = ER_MALLOC;
		else
		{
			if (DialogDo (hDialog, CENTER, CENTER, NULL, PDBuffer) == KEY_ENTER )
			{
				Flags = GetFlags ();
				Flags &= ~(FLAG_PASSWORD | FLAGS_AB);
				Flags |= FLAG_PASSWORD * (PDBuffer[0] == 1);
				PDBuffer[1]--;
				PDBuffer[1] <<= FLAG_AB_OFFSET;
				Flags |= (unsigned int)PDBuffer[1];
				if (PDBuffer[2] != oldDateformat) Ret = convertAddressbookDates (PDBuffer[2], oldDateformat);
				if (Ret == ER_SUCCESS)
				{
					SetFlags (Flags);
					writeDateFormat (PDBuffer[2]);
					if ( (PDBuffer[0] == 1) && (peek (&gr_Agenda->DataFile.Header->Password) == 0))
					{
						if (MsgBox (MA_TITLE, MSG_ER_PW_NOT_SET, BT_OK, BT_CANCEL) == OK)
							Ret = MenuChangePassword();
						else DeletePassword (1);				
					}
				}
			}
		}
	}
	
	HeapFreeHNULL (&hPopUpPW);
	HeapFreeHNULL (&hPopUpAB);
	HeapFreeHNULL (&hPopUpDF);
	HeapFreeHNULL (&hPopupPD);
	HeapFreeHNULL (&hDialog);
	
	return (Ret);
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MenuInfo ()
{
	int			Ret			= ER_SUCCESS;
	WINDOW 		*Win		= HeapAllocPtr (sizeof (WINDOW));
	TIdleLoop	idl;
	
	if (WinOpen (Win, MakeWinRect (INFO_WINRECT), WF_SAVE_SCR | WF_TTY))
	{
		WinActivate (Win);
		WinFont (Win, F_8x10);
		WinBitmapPut (Win, 10, 5, &AgendaIcon, A_REPLACE);
		WinStrXY (Win, 48, 7, CL_INFO_SHORT );
		WinFont (Win, F_6x8);
		WinStrXY (Win, 48, 20, MI_NAME);
		WinStrXY (Win, 48, 30, MI_EMAIL);
		WinStrXY (Win, (INFO_WIDTH - strlen(MI_L1)*6)/2-2, 52, MI_L1);
		WinStrXY (Win, (INFO_WIDTH - strlen(MI_L2)*6)/2-2, 62, MI_L2);
		WinStrXY (Win, (INFO_WIDTH - strlen(MI_L3)*6)/2-2, 72, MI_L3);
		WinLine (Win, MakeWinRect (INFO_LINE));
		stIdle();
		
		IdleLoop (&idl, ILF_NOMENU, NULL, NULL, NULL);
		WinClose (Win);
		HeapFreePtr (Win);
	}
	else Ret = ER_MALLOC;
		
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void DeletePassword (int ShowMsg)
{
	if (ShowMsg) MsgBox (MSG_ER_TITLE, MSG_ER_DISABLE_PW, BT_OK, BT_NONE);
	SetFlags (GetFlags () & ~FLAG_PASSWORD);
	poke (&gr_Agenda->DataFile.Header->Password, 0);
}


int MenuChangePassword ()
{
	int			Ret		= OK;
	char		PasswordA[9];
	char		PasswordB[9];
	
	PasswordA[0] = '\0';
	
	while ( (Ret == OK) && (strlen (PasswordA) < 4))
	{
		Ret = PasswordDlg (	DLG_PW_TITLE, DLG_ENTER_PW1, PasswordA );
		if ( (Ret == OK) && (strlen (PasswordA) < 4))
			MsgBox (MSG_ER_TITLE, MSG_ER_PW_SHORT, BT_OK, BT_NONE);
	}
	
	if (Ret == OK) Ret = PasswordDlg (	DLG_PW_TITLE, DLG_ENTER_PW2, PasswordB );
	if (Ret == OK)
	{
		if ( strcmp (PasswordA, PasswordB) )
		{
			MsgBox (MSG_ER_TITLE, MSG_ER_DIFFRENT_PW, BT_OK, BT_NONE);
			DeletePassword (0);
		}
		else
		{
			EncodeData (PasswordA, ENCODE_KEY_P, 8); 
			memcpy (&gr_Agenda->DataFile.Header->Password, PasswordA, 8);
			if ( !(GetFlags () & FLAG_PASSWORD) )
			{
				if (MsgBox (MA_TITLE, MSG_ER_PW_NOT_SEL, BT_YES, BT_NO) == OK)
					SetFlags (GetFlags () | FLAG_PASSWORD);
			}
		}
	}
	
	if (Ret == CANCEL) DeletePassword (1);

	return Ret;
}


// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
