// C Source File

/* --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AgndConv V.E oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Date:         November 2003 ooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------
// -- Last modification -- 15.XI.2003 --- 17:00 -----------------------------
// -- Built with TIGCC  -- 0.95 beta 3 --------------------------------------
// --------------------------------------------------------------------------
// -- Project Options   -- Calcs: V200 / 92Plus
//                      -- Optimize ROM Calls
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// ------------------------------------------------------------------------*/

#include "BuildOptions.h"
#include <tigcclib.h>

#include "Agndconv.h"
#include "V3.h"
#include "consts.h"
#include "V2.h"
#include "V4.h"
#include "VE.h"


// --------------------------------------------------------------------------
// --- replaces each charachter ch1 by ch2 in the string string. strreplce --
// --- returns the number of replacements. ----------------------------------
// --------------------------------------------------------------------------

int strreplce (char *string, char ch1, char ch2)
{
	int		n	= 0;
	
	while (*string != 0)
	{
		if (*string == ch1)
		{
			*string = ch2;
			n++;
		}
		string++;
	}
	
	return n;
}

// --------------------------------------------------------------------------
// --- returns a pointer to the string number num of 0-termed stringlist ----
// --- data. ----------------------------------------------------------------
// --------------------------------------------------------------------------

char *GetStrFromListEx (char *data, unsigned int num)
{
	unsigned int	i;
	char			*ptr;
	
	ptr = data;
	for (i = 0; (i < num); i++)
	{
		ptr += strlen (ptr)+1;
	}
		
	return ptr;
}

// --------------------------------------------------------------------------
// --- Encodes the Data with Key -------------------------------------------
// --------------------------------------------------------------------------

int EncodeData (char *Data, long int Key, unsigned int Num)
{
	unsigned int	n = 0;
	TEncodeKey		EncodeKey;
	
	EncodeKey.Key = Key;
	
	while (n < Num)
	{
		*Data ^= EncodeKey.Byte[n % 4];
		n++;
		Data++;
	}
	
	return (0);
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Shows a Messagebox ---------------------------------------------------
// ---	Text is a string with \0 chars after each line to display. ----------
// --------------------------------------------------------------------------

int MsgBox (char *Title, char *Text, short LeftButton, short RightButton )
{
	HANDLE			DlgHandle	= H_NULL;
	HANDLE			handle		= H_NULL;
	int				i;
	char			*ptr;
	unsigned int	maxlen		= 0;
	int				Ret			= ER_SUCCESS;
	int				Width;
	int				Rows		= 0;
	
	ptr = Text;
	while (*ptr != '\0')
	{
		if (strlen (ptr) > maxlen) maxlen = strlen (ptr);
		ptr += strlen (ptr)+1;
		Rows++;
	}
	Width = maxlen*6+20;
	if (Width < MSG_BOX_MIN_WIDTH) Width = MSG_BOX_MIN_WIDTH;
	DlgHandle = DialogNewSimple (Width, Rows*10+40);
	if (DlgHandle == H_NULL) Ret = ER_MALLOC;
	if (Ret == ER_SUCCESS)
	{
		handle = DialogAddTitle (DlgHandle, Title, LeftButton, RightButton);
		if (handle == H_NULL) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)
	{
		ptr = Text;
		for (i=0; i<Rows && handle; i++)
		{
			handle = DialogAddText (DlgHandle, 10, i*10+20, ptr);
			ptr += strlen (ptr)+1;
		}
		if (handle == H_NULL) Ret = ER_MALLOC;
		else
		{
			Ret = DialogDo (DlgHandle, CENTER, CENTER, NULL, NULL);
			if (Ret == KEY_ENTER) Ret = OK;
			else Ret = CANCEL;
			ST_busy (ST_IDLE);
		}
	}

	if (DlgHandle) HeapFree (DlgHandle);
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *GetSymnamePtr (char *Buffer, char *Folder, char *File)
{
	int				Size;
	char			Empty[] = "";
	
	if (Folder == NULL) Folder = Empty;
	memset (Buffer, 0, 19);
	strcpy (Buffer+1, Folder);
	strcat (Buffer+1, "\\"); 
	strcat (Buffer+1, File);
	Size = strlen (Buffer+1)+1;
	
	return (Buffer + Size);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

long int RoundUpEven (long int Num)
{
	if ( ((float)Num / 2.0) != ( Num / 2) ) Num++;
	
	return Num;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int ReDerefDataFile (TFile *File)
{
	File->DataPtr = HeapDeref (File->handle);
	File->Header = (TDataFileV3 *)File->DataPtr;

	return 0;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int ReallocDataFile (TFile *File, char *Pos, unsigned long NewSize, char **NewPos)
{
	unsigned int	OldSize;
	unsigned int	nToMove;
	int				Ret			= ER_SUCCESS;
	HANDLE			handle;
	long int		dPos;
	char			*OldFilePos;
	
	OldSize = File->Header->Size + 2;
	if (NewSize > OldSize)
	{
		OldFilePos = File->DataPtr;
		nToMove = OldSize - (Pos - File->DataPtr);
		HeapUnlock (File->handle);
		handle = HeapRealloc (File->handle, NewSize);
		HeapLock (File->handle);
		if (handle == H_NULL) Ret = ER_MALLOC;
		else
		{
			ReDerefDataFile (File);
			dPos = File->DataPtr - OldFilePos;
			Pos += dPos;
			memmove (Pos + NewSize - OldSize, Pos, nToMove);
		}			
	}
	else
	{
		nToMove = NewSize - (Pos - File->DataPtr);
		memmove (Pos, Pos + OldSize - NewSize, nToMove);
		handle = HeapRealloc (File->handle, NewSize);
		if (handle == H_NULL) Ret = ER_MALLOC;		
	}
	if (Ret == ER_SUCCESS) File->Header->Size = NewSize - 2;

	if (NewPos != NULL) *NewPos = Pos;

	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int OpenFile (HANDLE *handle)
{
	
	char		SymName[19];
	HSYM		hsym;
	int			Ret				= ER_SUCCESS;
	SYM_ENTRY	*SymEntry		= NULL;
	
	hsym = SymFind (GetSymnamePtr (SymName, NULL, (char *)"agnddata" ));
	if (hsym.folder)
	{
		SymEntry = DerefSym (hsym);
		if (SymEntry->flags.bits.archived) Ret = ER_DATAFILE_ARCHIVED;  
		*handle = SymEntry->handle;
	}
	else Ret = ER_SYM_NOT_FOUND;
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void _main(void)
{
	HANDLE			handle;
	int				Ret			= ER_SUCCESS;
	unsigned int	Version;
	unsigned int	Key;
	unsigned int	Size;
	char			ErrorBuf[50];
	
	if (HeapAvail () < (5*1024)) Ret = ER_MALLOC;
	else Ret = OpenFile (&handle);

	if (Ret == ER_SUCCESS)
	{
		Version = peek_w ((char *)HeapDeref (handle) + 2);
		Size = peek_w (HeapDeref (handle));
		
		Key = Version & 0xFF00;
		if ( (Key & 1) == 1 ) Version = 1;
		if ( ((Key != 0xFF00) && (Version != 1)) || (Size < 25) )
		{
			MsgBox ((char *)"Error", (char *)"This file doesn't have a correct\0agnddata format. It seems to be\0corrupt!\0", BT_OK, BT_NONE);
		}
		else
		{
			switch (Version & 0x00FF)
			{
				case 1:		if (Ret == ER_SUCCESS) Ret = Update1to2 (handle);

				case 2:		if (Ret == ER_SUCCESS) Ret = Update2to3 (handle);

				case 3:		if (Ret == ER_SUCCESS) Ret = Update3to4 (handle);

				case 4:		if (Ret == ER_SUCCESS) Ret = Update4toE (handle);
							break;

				case 5:		MsgBox ((char *)"Conversion", (char *)"This is already version E\0of agnddata!\0", BT_OK, BT_NONE);
							break;

				default:	MsgBox ((char *)"Error", (char *)"This version of agnddata is unknown.\0", BT_OK, BT_NONE);
			}
		}
	}
	
	switch (Ret)
	{
		case CANCEL:
		case OK:
		case ER_SUCCESS:
			break;
			
		case ER_DATAFILE_ARCHIVED:
			MsgBox ((char *)"Error", (char *)"Please unarchive agnddata!\0Conversion failed.\0", BT_OK, BT_NONE);
			break;
			
		case ER_SYM_NOT_FOUND:
			MsgBox ((char *)"Error", (char *)"agnddata not found in current\0folder!\0Conversion failed.\0", BT_OK, BT_NONE);
			break;
			
		case ER_MALLOC:
			MsgBox ((char *)"Error", (char *)"Not enough memory!\0Conversion failed.\0", BT_OK, BT_NONE);
			break;

		default:
			sprintf (ErrorBuf, "Error number %d occured. Conversion failed.", Ret);
			poke (ErrorBuf + 23, 0);
			poke (ErrorBuf + 43, 0); 
			MsgBox ((char *)"Error", ErrorBuf, BT_OK, BT_NONE);
	}
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------

